import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Enumeration;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicLong;

public class BugRewriteBatchedStatements517 {

	public static void main(String[] args) {

		String url = "jdbc:mysql://localhost:3306/test?user=root";
		if (args != null && args.length > 0) {
			url = args[0];
		}

		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
		}

		String testWithoutOnDuplicateKey = "INSERT INTO TEST_DUPLICATE(ID, S1, S2, S3, D1, D2, D3, N1, N2, N3) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		String testWithOnDuplicateKey = testWithoutOnDuplicateKey
				+ " ON DUPLICATE KEY UPDATE S1 = VALUES(S1), S2 = VALUES(S2), S3 = VALUES(S3), D1 = VALUES(D1), D2 = VALUES(D2), D3 = VALUES(D3), N1 = N1 + VALUES(N1), N2 = N2 + VALUES(N2), N2 = N2 + VALUES(N2)";

		Enumeration<Driver> de = DriverManager.getDrivers(); 
		Driver d = null;
		while (de.hasMoreElements()) {
			d = de.nextElement();
			if (d instanceof com.mysql.jdbc.Driver) {		
				com.mysql.jdbc.Driver dm = (com.mysql.jdbc.Driver) d;
				System.out.println("Driver " + dm.getMajorVersion() + "." + dm.getMinorVersion() + ": " + dm.toString());
			}
		}
		
		//-------------------------
		int threadCount = 1;		
		//-------------------------
		
		System.out.println("-----------------------------------------------------------------");
		System.out.println("Test with rewriteBatchedStatements and without 'ON DUPLICATE KEY'");
		System.out.println("-----------------------------------------------------------------");
		test(1, testWithoutOnDuplicateKey, url, true, threadCount);
		test(10, testWithoutOnDuplicateKey, url, true, threadCount);
		test(100, testWithoutOnDuplicateKey, url, true, threadCount);
		test(1000, testWithoutOnDuplicateKey, url, true, threadCount);
		System.out.println("-----------------------------------------------------------------");
		System.out.println("Test without rewriteBatchedStatements and without 'ON DUPLICATE KEY'");
		System.out.println("-----------------------------------------------------------------");
		test(1, testWithoutOnDuplicateKey, url, false, threadCount);
		test(10, testWithoutOnDuplicateKey, url, false, threadCount);
		test(100, testWithoutOnDuplicateKey, url, false, threadCount);
		test(1000, testWithoutOnDuplicateKey, url, false, threadCount);
		System.out.println("-----------------------------------------------------------------");
		System.out.println("Test with rewriteBatchedStatements and with 'ON DUPLICATE KEY'");
		System.out.println("-----------------------------------------------------------------");
		test(1, testWithOnDuplicateKey, url, true, threadCount);
		test(10, testWithOnDuplicateKey, url, true, threadCount);
		test(100, testWithOnDuplicateKey, url, true, threadCount);
		test(1000, testWithOnDuplicateKey, url, true, threadCount);
		System.out.println("-----------------------------------------------------------------");
		System.out.println("Test without rewriteBatchedStatements and with 'ON DUPLICATE KEY'");
		System.out.println("-----------------------------------------------------------------");
		test(1, testWithOnDuplicateKey, url, false, threadCount);
		test(10, testWithOnDuplicateKey, url, false, threadCount);
		test(100, testWithOnDuplicateKey, url, false, threadCount);
		test(1000, testWithOnDuplicateKey, url, false, threadCount);
		System.out.println("-----------------------------------------------------------------");
	}

	private static void test(final int test, final String query, final String url, final boolean rewriteBatchedStatements, int threadCount) {
		Connection conn = null;

		final Date d1 = new Date(System.currentTimeMillis());
		final Date d2 = new Date(System.currentTimeMillis() + 1000000);
		final Date d3 = new Date(System.currentTimeMillis() + 1250000);

		try {
			conn = DriverManager.getConnection(url + (rewriteBatchedStatements ? "&rewriteBatchedStatements=true&cachePreparedStatements=true" : "&cachePreparedStatements=true"));
			conn.setAutoCommit(false);
			conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);

			conn.createStatement().execute(
				"DROP TABLE IF EXISTS TEST_DUPLICATE");
			conn.createStatement().execute(
				"CREATE TABLE IF NOT EXISTS TEST_DUPLICATE ( ID INTEGER, S1 VARCHAR(100), S2 VARCHAR(100), S3 VARCHAR(100), D1 DATETIME, D2 DATETIME, D3 DATETIME, N1 DECIMAL(28,6), N2 DECIMAL(28,6), N3 DECIMAL(28,6), UNIQUE KEY UNIQUE_KEY_TEST_DUPLICATE (ID) )");

			
			final CountDownLatch waitToStart = new CountDownLatch(1);
			final CountDownLatch waitEnded = new CountDownLatch(threadCount);
			Thread[] threads = new Thread[threadCount];
			final AtomicLong tdiffTime = new AtomicLong(0);
			for (int j = 0; j < threadCount; ++j) {
				final int lj = j + 1;
				threads[j] = new Thread( new Runnable() {
					public void run() {
						Connection conn = null;
						try {
							conn = DriverManager.getConnection(url + (rewriteBatchedStatements ? "&rewriteBatchedStatements=true&cachePreparedStatements=false" : "&cachePreparedStatements=false"));
							conn.setAutoCommit(false);
							conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
							waitToStart.await();
							long startTime = 0L;
							long diffTime = 0L;
							for (int k = 1; k<= 10; ++k) {								
								PreparedStatement ptmt = conn.prepareStatement(query);
								for (int i = 0; i < test; i++) {
									ptmt.setObject(1, Integer.valueOf(i + (10 * lj * test) + ( k * test)), Types.INTEGER);
									ptmt.setObject(2, String.valueOf(i), Types.VARCHAR);
									ptmt.setObject(3, String.valueOf(i * 0.1), Types.VARCHAR);
									ptmt.setObject(4, String.valueOf(i / 3), Types.VARCHAR);
									ptmt.setObject(5, new Timestamp(d1.getTime()), Types.TIMESTAMP);
									ptmt.setObject(6, new Timestamp(d2.getTime()), Types.TIMESTAMP);
									ptmt.setObject(7, new Timestamp(d3.getTime()), Types.TIMESTAMP);
									ptmt.setObject(8, BigDecimal.valueOf(i + 0.1), Types.DECIMAL);
									ptmt.setObject(9, BigDecimal.valueOf(i * 0.1), Types.DECIMAL);
									ptmt.setObject(10, BigDecimal.valueOf(i / 3), Types.DECIMAL);
									ptmt.addBatch();
								}
								startTime = System.nanoTime();
								ptmt.executeBatch();
								ptmt.close();
								conn.commit();	
								diffTime += System.nanoTime()  - startTime;
							}
							tdiffTime.addAndGet(diffTime);
						} catch (SQLException e) {
							System.out.println("KO : " + e.getMessage());
						} catch (InterruptedException e) {
							System.out.println("KO : " + e.getMessage());
						} finally {
							if (conn != null) {
								try {
									conn.close();
								} catch (SQLException ignored) {
								}
							}
							waitEnded.countDown();
						}
					}
				});
				threads[j].setName("-" + j);
				threads[j].start();
			}

			
			waitToStart.countDown();
			try {
				waitEnded.await();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			long diffTime = tdiffTime.get();
			
			ResultSet rs = conn.createStatement().executeQuery(
					"SELECT COUNT(*) FROM TEST_DUPLICATE");
			if (rs != null && rs.next() ) {
				System.out.println((diffTime / 1000000) 
						+ " ms, "
						+ rs.getInt(1) 
						+ " rows -> 10 executeBatch *  " 
						+ threadCount
						+ " threads ( " 
						+ test + " batch/exec) = " 
						+ ( ((long)( (long) rs.getInt(1) * 1000000000)) / diffTime )
						+ " IPS " );
			} else {
				System.out.println("KO," + test + " by batch: " + test + ","
						+ rewriteBatchedStatements);
			}
			conn.createStatement().execute(
					"DROP TABLE IF EXISTS TEST_DUPLICATE");
		} catch (SQLException ex) {
			System.out.println("KO : " + ex.getMessage());
		} finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException ignored) {
				}
			}
		}
	}
}