import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

public class BugRewriteBatchedStatementsTestFix {

	public static void main(String[] args) {

		String url = "jdbc:mysql://localhost:3306/test?user=root";
		if (args != null && args.length > 0) {
			url = args[0];
		}

		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
		}

		test(url);
	}

	private static void test(String url) {
		Connection conn = null;

		try {
			conn = DriverManager.getConnection(url + "&rewriteBatchedStatements=true&cachePreparedStatements=true");
			conn.setAutoCommit(false);
			conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);

			conn.createStatement().execute(
				"DROP TABLE IF EXISTS TEST_DUPLICATE");
			conn.createStatement().execute(
				"CREATE TABLE IF NOT EXISTS TEST_DUPLICATE ( ID INTEGER, S1 VARCHAR(100), UNIQUE KEY UNIQUE_KEY_TEST_DUPLICATE (ID) )");

			int delta = 0;
			int[] result = null;
			
			System.out.println("----------------------");
			
			String query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( ?, ? )";
			System.out.print(query);
			PreparedStatement ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + delta), Types.INTEGER);
				ptmt.setObject(2, String.valueOf(i), Types.VARCHAR);
				ptmt.addBatch();
			}
			result = result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			delta += 10;
			checkTabnleSize(conn, delta);
					
			query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( ?, '10' )";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + delta), Types.INTEGER);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			delta += 10;
			checkTabnleSize(conn, delta);
			
			query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( 1, '10' ) ON DUPLICATE KEY UPDATE S1 = VALUES(S1)";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			checkTabnleSize(conn, delta);
			
			query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( ?, '10' ) ON DUPLICATE KEY UPDATE S1 = VALUES(S1)";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + delta), Types.INTEGER);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			delta += 10;
			checkTabnleSize(conn, delta);
			
		    query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( ?, ? ) ON DUPLICATE KEY UPDATE S1 = '10'";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + delta), Types.INTEGER);
				ptmt.setObject(2, String.valueOf(i), Types.VARCHAR);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			delta += 10;
			checkTabnleSize(conn, delta);
			
			
			query = "INSERT INTO TEST_DUPLICATE (ID, S1) VALUES ( ?, ? ) ON DUPLICATE KEY UPDATE S1 = ?";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + delta), Types.INTEGER);
				ptmt.setObject(2, String.valueOf(i), Types.VARCHAR);
				ptmt.setObject(3, String.valueOf(i + 1), Types.VARCHAR);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			System.out.println(" -> OK");
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			delta += 10;
			checkTabnleSize(conn, delta);
			
			
			System.out.println("----------------------");
			
			
			
			query = "DELETE FROM TEST_DUPLICATE WHERE ID = 15000";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();	
			for (int i : result) {	
				if (i != 0) {					
					System.out.println("KO: one result is wrong");
				}
			}
			checkTabnleSize(conn, delta);
			System.out.println(" -> OK");
			
			query = "DELETE FROM TEST_DUPLICATE WHERE ID = ?";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i), Types.INTEGER);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();
			checkTabnleSize(conn, delta - 10);
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			System.out.println(" -> OK");
			
			
			query = "DELETE FROM TEST_DUPLICATE WHERE ID = ? AND S1 != ? AND S1 NOT LIKE '%?%'";
			System.out.print(query);
			ptmt = conn.prepareStatement(query);
			for (int i = 0; i < 10; i++) {
				ptmt.setObject(1, Integer.valueOf(i + 10), Types.INTEGER);
				ptmt.setObject(2, "TOTO", Types.VARCHAR);
				ptmt.addBatch();
			}
			result = ptmt.executeBatch();
			ptmt.close();
			conn.commit();
			for (int i : result) {	
				if (i != 1) {					
					System.out.println("KO: one result is wrong");
				}
			}
			System.out.println(" -> OK");
			checkTabnleSize(conn, delta - 20);
			
			
			System.out.println("----------------------");
			
			conn.createStatement().execute(
					"DROP TABLE IF EXISTS TEST_DUPLICATE");
		} catch (SQLException ex) {
			System.out.println("KO : " + ex.getMessage());
		} finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException ignored) {
				}
			}
		}
	}
	
	private static void checkTabnleSize(Connection conn, int size) throws SQLException {
		ResultSet rs = null;
		try {
			rs = conn.createStatement().executeQuery("SELECT COUNT(*) FROM TEST_DUPLICATE");
			if (rs == null || !rs.next()) {
				System.out.println("KO: nb rows in db is wrong");
			} else if (rs.getInt(1) != size) {
				System.out.println("KO: nb rows in db is wrong, in db = " + rs.getInt(1)  + ", expect size = " + size);
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
		}
	}
}