import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Enumeration;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicLong;

public class BugRewriteBatchedStatementsUpdateTest {

	public static void main(String[] args) {

		String url = "jdbc:mysql://localhost:3306/test?user=root";
		if (args != null && args.length > 0) {
			url = args[0];
		}

		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
		}

		Enumeration<Driver> de = DriverManager.getDrivers(); 
		Driver d = null;
		while (de.hasMoreElements()) {
			d = de.nextElement();
			if (d instanceof com.mysql.jdbc.Driver) {		
				com.mysql.jdbc.Driver dm = (com.mysql.jdbc.Driver) d;
				System.out.println("Driver " + dm.getMajorVersion() + "." + dm.getMinorVersion() + ": " + dm.toString());
			}
		}
		
		Connection conn = null;
		try {
			conn = DriverManager.getConnection(url + "&cachePreparedStatements=true");
			conn.setAutoCommit(false);
			conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);

			conn.createStatement().execute(
				"DROP TABLE IF EXISTS TEST_DUPLICATE");
			conn.createStatement().execute(
				"CREATE TABLE IF NOT EXISTS TEST_DUPLICATE ( ID INTEGER, S1 VARCHAR(100), S2 VARCHAR(100), S3 VARCHAR(100), D1 DATETIME, D2 DATETIME, D3 DATETIME, N1 DECIMAL(28,6), N2 DECIMAL(28,6), N3 DECIMAL(28,6), UNIQUE KEY UNIQUE_KEY_TEST_DUPLICATE (ID) )");

			final Date d1 = new Date(System.currentTimeMillis());
			final Date d2 = new Date(System.currentTimeMillis() + 1000000);
			final Date d3 = new Date(System.currentTimeMillis() + 1250000);
			
			for (int j =0; j < 100; j++) {
				PreparedStatement ptmt = conn.prepareStatement("INSERT INTO TEST_DUPLICATE(ID, S1, S2, S3, D1, D2, D3, N1, N2, N3) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				for (int i = 0; i < 100; i++) {
					ptmt.setObject(1, Integer.valueOf(i + (j * 100)), Types.INTEGER);
					ptmt.setObject(2, String.valueOf(i), Types.VARCHAR);
					ptmt.setObject(3, String.valueOf(i * 0.1), Types.VARCHAR);
					ptmt.setObject(4, String.valueOf(i / 3), Types.VARCHAR);
					ptmt.setObject(5, new Timestamp(d1.getTime()), Types.TIMESTAMP);
					ptmt.setObject(6, new Timestamp(d2.getTime()), Types.TIMESTAMP);
					ptmt.setObject(7, new Timestamp(d3.getTime()), Types.TIMESTAMP);
					ptmt.setObject(8, BigDecimal.valueOf(i + 0.1), Types.DECIMAL);
					ptmt.setObject(9, BigDecimal.valueOf(i * 0.1), Types.DECIMAL);
					ptmt.setObject(10, BigDecimal.valueOf(i / 3), Types.DECIMAL);
					ptmt.addBatch();
				}
				ptmt.executeBatch();
				ptmt.close();
				conn.commit();
				System.out.print(".");
			}
			System.out.println("");
			
			
			String query = "UPDATE TEST_DUPLICATE SET S1 = ?, S2 = ?, S3 = ?, D1 = ?, D2 = ?, D3 = ?, N1 = ?, N2 = ?, N3 = ? WHERE ID = ?";
			//-------------------------
			int threadCount = 1;		
			//-------------------------		
			System.out.println(query);
			System.out.println("-------------------------------------");
			System.out.println("Test with rewriteBatchedStatements   ");
			System.out.println("-------------------------------------");
			test(1, query, url, true, threadCount);
			test(10, query, url, true, threadCount);
			test(100, query, url, true, threadCount);
			test(1000, query, url, true, threadCount);
			System.out.println("-------------------------------------");
			System.out.println("Test without rewriteBatchedStatements");
			System.out.println("-------------------------------------");
			test(1, query, url, false, threadCount);
			test(10, query, url, false, threadCount);
			test(100, query, url, false, threadCount);
			test(1000, query, url, false, threadCount);
			System.out.println("-------------------------------------");
			
			conn.createStatement().execute(
			"DROP TABLE IF EXISTS TEST_DUPLICATE");
		} catch (SQLException ex) {
			System.out.println("KO : " + ex.getMessage());
		} finally {
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException ignored) {
				}
			}
		}
	}

	private static void test(final int test, final String query, final String url, final boolean rewriteBatchedStatements, int threadCount) {

		final Date d1 = new Date(System.currentTimeMillis());
		final Date d2 = new Date(System.currentTimeMillis() + 1000000);
		final Date d3 = new Date(System.currentTimeMillis() + 1250000);

		final CountDownLatch waitToStart = new CountDownLatch(1);
		final CountDownLatch waitEnded = new CountDownLatch(threadCount);
		Thread[] threads = new Thread[threadCount];
		final AtomicLong tdiffTime = new AtomicLong(0);
		for (int j = 0; j < threadCount; ++j) {
			final int lj = j + 1;
			threads[j] = new Thread( new Runnable() {
				public void run() {
					Connection conn = null;
					try {
						conn = DriverManager.getConnection(url + (rewriteBatchedStatements ? "&rewriteBatchedStatements=true&cachePreparedStatements=false" : "&cachePreparedStatements=true"));
						conn.setAutoCommit(false);
						conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
						waitToStart.await();
						long startTime = 0L;
						long diffTime = 0L;
						for (int k = 1; k<= 10; ++k) {								
							PreparedStatement ptmt = conn.prepareStatement(query);
							for (int i = 0; i < test; i++) {
								ptmt.setObject(1, String.valueOf(i), Types.VARCHAR);
								ptmt.setObject(2, String.valueOf(i * 0.1), Types.VARCHAR);
								ptmt.setObject(3, String.valueOf(i / 3), Types.VARCHAR);
								ptmt.setObject(4, new Timestamp(d1.getTime()), Types.TIMESTAMP);
								ptmt.setObject(5, new Timestamp(d2.getTime()), Types.TIMESTAMP);
								ptmt.setObject(6, new Timestamp(d3.getTime()), Types.TIMESTAMP);
								ptmt.setObject(7, BigDecimal.valueOf(i + 0.1), Types.DECIMAL);
								ptmt.setObject(8, BigDecimal.valueOf(i * 0.1), Types.DECIMAL);
								ptmt.setObject(9, BigDecimal.valueOf(i / 3), Types.DECIMAL);
								ptmt.setObject(10, Integer.valueOf(i + (10 * lj * test) + ( k * test)), Types.INTEGER);
								ptmt.addBatch();
							}
							startTime = System.nanoTime();
							ptmt.executeBatch();
							ptmt.close();
							conn.commit();	
							diffTime += System.nanoTime()  - startTime;
						}
						tdiffTime.addAndGet(diffTime);
					} catch (SQLException e) {
						System.out.println("KO : " + e.getMessage());
					} catch (InterruptedException e) {
						System.out.println("KO : " + e.getMessage());
					} finally {
						if (conn != null) {
							try {
								conn.close();
							} catch (SQLException ignored) {
							}
						}
						waitEnded.countDown();
					}
				}
			});
			threads[j].setName("-" + j);
			threads[j].start();
		}	
		waitToStart.countDown();
		try {
			waitEnded.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		long diffTime = tdiffTime.get();
		
		System.out.println((diffTime / 1000000) 
				+ " ms, "
				+ (test * 10 * threadCount)
				+ " query = 10 executeBatch *  " 
				+ threadCount
				+ " threads ( " 
				+ test + " batch/exec) = " 
				+ ( ((long)( (long) (test * 10 * threadCount) * 1000000000)) / diffTime )
				+ " QPS " );
	}
}